//using namespace RooFit;
#include "TF1.h"
#include "Math/WrappedTF1.h"
#include "Math/GaussIntegrator.h"


double CrystalBall(double* x, double* par){
//http://en.wikipedia.org/wiki/Crystal_Ball_function
double xcur= x[0];
double alpha = par[0];
double n = par[1];
double mu = par[2];
double sigma = par[3];
double N = par[4];
TF1*exp= new TF1("exp","exp(x)",1e-20,1e20);
double A; double B;
if (alpha < 0){
A =pow((n/(-1*alpha)),n)*exp->Eval((-1)*alpha*alpha/2);
B = n/(-1*alpha) + alpha;}
else {
A =pow((n/alpha),n)*exp->Eval((-1)*alpha*alpha/2);
B = n/alpha - alpha;}
double f;
if ((xcur-mu)/sigma > (-1)*alpha)
f = N*exp->Eval((-1)*(xcur-mu)*(xcur-mu)/(2*sigma*sigma));
else
f = N*A*pow((B- (xcur-mu)/sigma),(-1*n));
delete exp;
return f;
}


void CrystalBall(){
int min=-10, max=10;
TCanvas* crystalCanvas= new TCanvas("crystalCanvas","Heslo ke dnu kleslo",1024,1024);
TF1* crystal1Func= new TF1("cryslal1Func",&CrystalBall,min,max,30);
TF1* crystal2Func= new TF1("cryslal2Func",&CrystalBall,min,max,30);
TF1* crystal3Func= new TF1("cryslal3Func",&CrystalBall,min,max,30);
gStyle->SetOptTitle(0);
crystal1Func->SetLineStyle(1);
crystal1Func->SetLineColor(1);
crystal2Func->SetLineStyle(2);
crystal2Func->SetLineColor(2);
crystal3Func->SetLineStyle(7);
crystal3Func->SetLineColor(55);


crystal1Func->SetParameters(1,1,1,1,2);
crystal1Func->Draw();
crystal2Func->SetParameters(1,1,1,1.5,2);
crystal2Func->Draw("same");
crystal3Func->SetParameters(1,1,1,2,2);
crystal3Func->Draw("same");

double crystal1= crystal1Func->Integral(min,max);
double crystal2= crystal2Func->Integral(min,max);
double crystal3= crystal3Func->Integral(min,max);

TLegend *legend = new TLegend(.75,.75,.95,.95);
legend->AddEntry(crystal1Func,"#sigma =1.0");
legend->AddEntry(crystal2Func,"#sigma =1.5");
legend->AddEntry(crystal3Func,"#sigma =2.0");

TLegend *leg = new TLegend(.15,.95,.45,.7);
leg->AddEntry((TObject*)0, Form("Integer #sigma(1.0)=%4.2f", crystal1), "");
leg->AddEntry((TObject*)0, Form("Integer #sigma(1.5)=%4.2f", crystal2), "");
leg->AddEntry((TObject*)0, Form("Integer #sigma(2.0)=%4.2f", crystal3), "");
legend->Draw("same");
crystal1Func->GetHistogram()->GetYaxis()->SetTitle("f(x)");
crystal1Func->GetHistogram()->GetXaxis()->SetTitle("x");
leg->Draw("same");

cout<<"Crystal1="<<crystal1<<endl;
cout<<"Crystal2="<<crystal2<<endl;
cout<<"Crystal3="<<crystal3<<endl;


crystalCanvas->SaveAs("sigma.jpg");

crystal1Func->SetParameters(1,1,1,1,2);
crystal1Func->Draw();
crystal2Func->SetParameters(1.5,1,1,1,2);
crystal2Func->Draw("same");
crystal3Func->SetParameters(2,1,1,1,2);
crystal3Func->Draw("same");

double crystal1= crystal1Func->Integral(min,max);
double crystal2= crystal2Func->Integral(min,max);
double crystal3= crystal3Func->Integral(min,max);



TLegend *legend = new TLegend(.75,.75,.95,.95);
legend->AddEntry(crystal1Func,"#alpha =1");
legend->AddEntry(crystal2Func,"#alpha =1.5");
legend->AddEntry(crystal3Func,"#alpha =2");
TLegend *leg = new TLegend(.15,.95,.45,.7);
leg->AddEntry((TObject*)0, Form("Integer #alpha(1.0)=%4.2f", crystal1), "");
leg->AddEntry((TObject*)0, Form("Integer #alpha(1.5)=%4.2f", crystal2), "");
leg->AddEntry((TObject*)0, Form("Integer #alpha(2.0)=%4.2f", crystal3), "");
legend->Draw("same");
crystal1Func->GetHistogram()->GetYaxis()->SetTitle("f(x)");
crystal1Func->GetHistogram()->GetXaxis()->SetTitle("x");
leg->Draw("same");

crystalCanvas->SaveAs("alfap.jpg");

crystal1Func->SetParameters(1,1,1,1,2);
crystal1Func->Draw();
crystal2Func->SetParameters(1,2,1,1,2);
crystal2Func->Draw("same");
crystal3Func->SetParameters(1,3,1,1,2);
crystal3Func->Draw("same");

double crystal1= crystal1Func->Integral(min,max);
double crystal2= crystal2Func->Integral(min,max);
double crystal3= crystal3Func->Integral(min,max);


TLegend *legend = new TLegend(.75,.75,.95,.95);
legend->AddEntry(crystal1Func,"n =1");
legend->AddEntry(crystal2Func,"n =2");
legend->AddEntry(crystal3Func,"n =3");
TLegend *leg = new TLegend(.15,.95,.45,.7);
leg->AddEntry((TObject*)0, Form("Integer n(1)=%4.2f", crystal1), "");
leg->AddEntry((TObject*)0, Form("Integer n(2)=%4.2f", crystal2), "");
leg->AddEntry((TObject*)0, Form("Integer n(3)=%4.2f", crystal3), "");
legend->Draw("same");
crystal1Func->GetHistogram()->GetYaxis()->SetTitle("f(x)");
crystal1Func->GetHistogram()->GetXaxis()->SetTitle("x");
leg->Draw("same");

crystalCanvas->SaveAs("n.jpg");

return;
}

