
using namespace RooFit;

//plotting variables
class PlotsModule;
PlotsModule *mod;
Int_t cpd; Float_t bmg, lmg, tmg, ofs;
TLegend *leg;
TCanvas *c3 = new TCanvas("c3", "Analysis", 768., 768.);

//analysis variables
Double_t mjpsi, ptmax=999., ymin=-2.7, ymax=-1.2, alphafix=-999., nfix=-999., nbgfix[2]={200,200}, mmin=-999., mmax=999., binsiz=-1., vysl, err, nex, nexr, sigma;
Bool_t makeM, makepT; Char_t *datatxt; Char_t *signaltxt; Char_t *ggtxt=""; Char_t *bgtxt=""; Char_t *gPbtxt=""; Char_t *lstxt="";
UShort_t mode=1;
Char_t *plottitle; Char_t *bgtitle="";
Int_t nbinsM, nbinspT;

//roofit
RooRealVar m("m", "dilepton mass [GeV]", 2., 5.) ;
RooRealVar y("y", "dilepton rapidity", -2.7, -1.0) ;
RooRealVar pt("pt", "dilepton pT [GeV]", 0., 10.) ;

//_____________________________________________________________________________
void Set_pPb() {

  makeM   = 1;
  makepT  = 0;

  mode = 1; // mode for the pT fit,  0 - toyMC for bkg,  1 - J/psi fit,  2 - gamma-gamma fit

  
  datatxt   = "TXT/sel1/aod_run0/pTyM_lhc13de_run0_all.txt";
  signaltxt = "TXT/sel1/esdmc_fd/pTyM_lhc13e6a_esd.txt";
  ggtxt     = "TXT/sel1/esdmc_fd/pTyM_lhc13e6f_esd.txt";
  bgtxt     = "TXT/sel1/aod_run0/pTyM_lhc13de_run0_all_zdcOR_as.txt";
  lstxt     = "TXT/sel1/aod_run0/pTyM_lhc13de_run0_all_ls.txt";

  //datatxt = signaltxt;
  //bgtxt = lstxt; lstxt="";

  // v0cNspd_as

  //general controls
  ymin      = _YMIN_;
  ymax      = _YMAX_;
  ptmax     = 1.95;
  binsiz    = 0.15;

  //mass fit
  alphafix  =_ALPHA_;   
  nfix      = 4.;
  nbinsM    = 60;

  //pT fit
  nbinspT   = 11;  // 12  14  23
  nbgfix[0] = 95;
  nbgfix[1] = 4;

  //gamma-gamma pT fit
  mmin = 2.9;
  mmax = 3.2;

  plottitle = "p-Pb #rightarrow p-Pb +J/#psi  #sqrt{s_{NN}} = 5.02 TeV";
  //plottitle = "p-Pb #gamma#gamma #rightarrow #mu^{+}#mu^{-}  #sqrt{s_{NN}} = 5.02 TeV";
  //plottitle = "p-Pb #rightarrow p-Pb +J/#psi, bkg by method 1 (ZNC & ZPC)";
  //plottitle = "p-Pb #rightarrow p-Pb +J/#psi, bkg by method 2 (ZNC & ZPC & !V0C)";
  //bgtitle = "(bkg by V0C)";
  //plottitle = "p-Pb #rightarrow p-Pb +J/#psi + ZNC+ZPC";
  //plottitle = "p-Pb #rightarrow p-Pb +J/#psi + ZNC+ZPC+!V0C";
  //plottitle = "p-Pb #rightarrow p-Pb +J/#psi + V0C";
  //plottitle = "MC J/#psi#rightarrow#mu^{+}#mu^{-} in #gamma-p, LHC13e6a, #sqrt{s_{NN}} = 5.02 TeV";



}//Set_pPb

//_____________________________________________________________________________
void Set_Pbp() {

  makeM   = 0;
  makepT  = 1;

  mode = 1; // mode for the pT fit,  0 - toyMC for bkg,  1 - J/psi fit,  2 - gamma-gamma fit

  datatxt   = "TXT/sel1/aod_run0/pTyM_lhc13f_run0_all.txt";
  signaltxt = "TXT/sel1/esdmc_fd/pTyM_lhc13f1a_esd.txt";
  ggtxt     = "TXT/sel1/esdmc_fd/pTyM_lhc13f1f_esd.txt";
  bgtxt     = "TXT/sel1/aod_run0/pTyM_lhc13f_run0_all_zdcOR_as.txt";
  lstxt     = "TXT/sel1/aod_run0/pTyM_lhc13f_run0_all_ls.txt";

  //datatxt = signaltxt;
  //bgtxt = gPbtxt;

  //general controls
  ymin      = -2.5;
  ymax      = -1.2;
  ptmax     = 1.8;
  binsiz    = 0.2;

  //mass fit
  alphafix  = 0.610;
  nfix      = 10.710;
  nbinsM    = 60;

  //pT fit
  nbinspT   = 14;
  //nbgfix[0] = 15;
  //nbgfix[1] = 1;

  //gamma-gamma pT fit
  mmin = 2.9;
  mmax = 3.2;

  plottitle = "Pb-p #rightarrow Pb-p +J/#psi  #sqrt{s_{NN}} = 5.02 TeV";
  //plottitle = "Pb-p #gamma#gamma #rightarrow #mu^{+}#mu^{-}  #sqrt{s_{NN}} = 5.02 TeV";
  //plottitle = "Pb-p #rightarrow Pb-p +J/#psi + ZNA+ZPA  #sqrt{s_{NN}} = 5.02 TeV";
  //plottitle = "MC J/#psi#rightarrow#mu^{+}#mu^{-} in #gamma-p, LHC13f1a, #sqrt{s_{NN}} = 5.02 TeV";

}//Set_Pbp

//_____________________________________________________________________________
void FitTemplates() {

  Set_pPb();
  //Set_Pbp();

  if(binsiz > 0.) {
    nbinspT = (Int_t) TMath::Ceil( ptmax/binsiz ); // round-up value
    ptmax = (Double_t) binsiz*nbinspT; // move max up to pass the bins
    cout<<"I-FitTemplates, using binsiz = "<<binsiz<<" , nbinspT = "<<nbinspT<<" and ptmax = "<<ptmax<<endl;
  }

  TDatabasePDG *pdgdat = TDatabasePDG::Instance();
  TParticlePDG *partP = pdgdat->GetParticle( 443 );
  mjpsi = partP->Mass();

  //load plots task
  gROOT->LoadMacro("PlotsModule.cxx+g");
  mod = new PlotsModule();
  gStyle->SetOptStat("");//nemruo
  gStyle->SetPalette(1);
  gStyle->SetLineWidth(2);      //axis line
  gStyle->SetFrameLineWidth(2); //frame line
  TGaxis::SetMaxDigits(3);
  //canvas parameters
  bmg = 0.15; // bottom margin
  lmg = 0.15; // left margin
  ofs = 1.3; // Y title offset

  if(makeM) FitMass();

  if(makepT) {
    RooDataSet* dataPtYM = RooDataSet::read(datatxt, RooArgList(pt,y,m), "Q") ;
    RooDataSet* signalPtYM = RooDataSet::read(signaltxt, RooArgList(pt,y,m), "Q") ;
    RooDataSet* ggPtYM=0x0;
    if(strcmp(ggtxt,"")!=0) ggPtYM = RooDataSet::read(ggtxt, RooArgList(pt,y,m), "Q") ;
    RooDataSet* bgPtYM =0x0;
    if(strcmp(bgtxt,"")!=0) bgPtYM = RooDataSet::read(bgtxt, RooArgList(pt,y,m), "Q") ;
    RooDataSet* gPbPtYM=0x0;
    if(strcmp(gPbtxt,"")!=0) gPbPtYM = RooDataSet::read(gPbtxt, RooArgList(pt,y,m), "Q") ;
    RooDataSet* lsPtYM=0x0;
    if(strcmp(lstxt,"")!=0) lsPtYM = RooDataSet::read(lstxt, RooArgList(pt,y,m), "Q");

    if(mode == 0) DrawInput(bgPtYM);
    if(mode == 1) FitPtJPsi(dataPtYM, bgPtYM, ggPtYM, signalPtYM, nbgfix, "basic",gPbPtYM);
    if(mode == 2) FitPtGG(dataPtYM, bgPtYM, ggPtYM, lsPtYM, nbgfix, "basic");
  }

  if(binsiz > 0.) {
    cout<<"I-FitTemplates, used binsiz = "<<binsiz<<" , nbinspT = "<<nbinspT<<" and ptmax = "<<ptmax<<endl;
  }

}//FitTemplates

//_____________________________________________________________________________
void FitMass() {

  RooDataSet* dataPtYMPt2 = RooDataSet::read(datatxt, RooArgList(pt,y,m), "Q") ;
  RooDataSet* lsPtYM=0x0;
  if(strcmp(lstxt,"")!=0) lsPtYM = RooDataSet::read(lstxt, RooArgList(pt,y,m), "Q");

  //RooDataSet *data = dataPtYMPt2;
  Char_t strcut[1024];
  sprintf(strcut, "y>%f && y<%f && pt<%f", ymin, ymax, ptmax);
  RooDataSet *data = dataPtYMPt2->reduce(strcut);
  if(lsPtYM) RooDataSet *lsdata = lsPtYM->reduce(strcut);

  // definition of Crystal Ball
  RooRealVar m0("m0","mass",mjpsi,3.05,3.15) ;
  RooRealVar n("n","n",4.0,0.,20.); // 0 10
  if(nfix > 0.) {n.setMin(nfix); n.setMax(nfix);}
  RooRealVar sig("sig","resolution",0.08,0.01,0.2);
  RooRealVar alpha("alpha","alpha",0.3,0.1,1.1);
  if(alphafix > 0.) {alpha.setMin(alphafix); alpha.setMax(alphafix);}
  RooCBShape cb("cb","Crystal Ball PDF",m,m0,sig,alpha,n) ;

  // create a PDF to describe the background
  RooRealVar lambda("lambda","Exponent",-1,-10,0) ;
  RooExponential* Bkgd= new RooExponential("Bkgd","Exponential",m,lambda);

  // Combined Pdf and fit
  Int_t nEvts = data->numEntries();
  RooRealVar nCB("nCB","Number of CB events",0.8*nEvts,0,nEvts);
  RooRealVar nBG("nBG","Number of BG events",0.2*nEvts,0,nEvts);
  if(strcmp(ggtxt,"")!=0) RooAddPdf CbBG("CbBG","Crystal Ball plus Background PDF", RooArgList(cb,*Bkgd), RooArgList(nCB,nBG));
  if(strcmp(ggtxt,"")==0) RooAddPdf CbBG("CbBG","Crystal Ball", RooArgList(cb), RooArgList(nCB));
  RooAddPdf BG("BG","Background PDF", RooArgList(*Bkgd), RooArgList(nBG));
  RooFitResult *r = CbBG.fitTo(*data, Extended(kTRUE), Save());
  //RooFitResult *r = BG.fitTo(*data, Extended(kTRUE), Save());

  m.setRange("JPsiMassRange",2.9,3.2);
  //RooAbsReal *iSig = CbBG.createIntegral(RooArgSet(*Bkgd), Range("JPsiMassRange"));
  RooAbsReal *iBG = Bkgd->createIntegral(m,NormSet(m),Range("JPsiMassRange"));
  RooAbsReal *iCB = cb.createIntegral(m,NormSet(m),Range("JPsiMassRange"));
  RooAbsReal *iCbBG = CbBG.createIntegral(m,NormSet(m),Range("JPsiMassRange"));

  Char_t sigstr[1024], bkgstr[1024];
  vysl = iCB->getVal()*nCB.getVal();
  err = iCB->getVal()*nCB.getError();
  nex = iBG->getVal()*nBG.getVal();
  nexr = iBG->getVal()*nBG.getError();
  sigma= sig.getVal();
	//Double_t test=1;
	//test= nCB->getVal()*nCB.getVal();

  //number of background
  sprintf(bkgstr, "N_{exp} = %.0f #pm %.0f", iBG->getVal()*nBG.getVal(), iBG->getVal()*nBG.getError());
  //number of J/psi candidates
  sprintf(sigstr, "N_{cb} = %.0f #pm %.0f", iCB->getVal()*nCB.getVal(), iCB->getVal()*nCB.getError());

  cout<<"######################"<<endl;
  //cout<<iCbBG->getVal()<<endl;//*nCB.getVal()*nBG.getVal()<<endl;
  //cout<<iCB->getVal()<<endl;
  //cout<<iCB->getVal()*nCB.getVal()+iBG->getVal()*nBG.getVal()<<endl;
  cout<<"      N(J/psi):  "<<Form("%.0f", iCB->getVal()*nCB.getVal())<<" +/- "<<Form("%.0f", iCB->getVal()*nCB.getError());
  cout<<" ,    Nexp:  "<<Form("%.0f", iBG->getVal()*nBG.getVal())<<" +/- "<<Form("%.0f", iBG->getVal()*nBG.getError())<<endl;;
  cout<<"      Alpha:     "<<alpha.getVal()<<" +/- "<<alpha.getError();
  cout<<" ,    M(J/psi):  "<<m0.getVal()<<" +/- "<<m0.getError();
  cout<<" ,    Sigma:  "<<sig.getVal()<<" +/- "<<sig.getError()<<endl;
  cout<<"      n:  "<<n.getVal()<<" +/- "<<n.getError()<<" ,    Lambda:    "<<lambda.getVal()<<" +/- "<<lambda.getError()<<endl;
  //cout<<sigstr<<endl;
  //cout<<bkgstr<<endl;
  //cout<<lstr<<endl;
  cout<<"######################"<<endl;
//return 0;

  if(lsPtYM) {
    TH1F *lsH = new TH1F("lsH","lsH", nbinsM, m.getMin(), m.getMax());
    for (Int_t i=0; i<lsdata->numEntries(); i++) lsH->Fill(lsdata->get(i)->getRealValue("m"));
    lsH->SetFillColor(kRed+1);
    lsH->SetLineColor(lsH->GetFillColor());
  }

  TCanvas *can = c3; //FIXME
  mod->AdjustCanvas(can, 1, 1, bmg, lmg);
  gPad->SetLeftMargin(lmg);

  RooPlot* frame = m.frame(nbinsM);
  mod->FrameProperties(frame);
  frame->GetYaxis()->SetTitleOffset(ofs) ;
  frame->SetTitle(plottitle);
  //frame->SetTitle("MC #gamma#gamma#rightarrow#mu^{+}#mu^{-}, LHC13e6f, #sqrt{s_{NN}} = 5.02 TeV");
  frame->SetXTitle("#mu^{+}#mu^{-} inv. mass [GeV/c^{2}]");
  //frame->SetYTitle("Events / (0.05 GeV/c^{2})");


  data->plotOn(frame);
  //cb.plotOn(frame, Precision(1e-4)) ; 
  CbBG.plotOn(frame, Precision(1e-6)) ; 
  if(strcmp(ggtxt,"")!=0) CbBG.plotOn(frame,Components(RooArgSet(*Bkgd)),LineColor(kRed),LineStyle(kDashed));
  if(strcmp(ggtxt,"")==0) CbBG.plotOn(frame);

  //BG.plotOn(frame,Components(RooArgSet(*Bkgd)),LineColor(kRed),LineStyle(kDashed));

  //CbBG.plotOn(frame,Components(RooArgSet(cb)),LineColor(kBlue),LineStyle(kDashed));
  //CbBG.paramOn(frame,data);

  Float_t lrn[5] = {0.54, 0.01, 0.55, 0.32, -1.};  //legend range, x, dx, y, dy, margin
  leg = mod->PrepareLegend(lrn, 0.035);
  //leg->AddEntry(data, "ALICE", "p");
  //leg->AddEntry((TObject*)0, "-2.7 < y < -1.2", "");
  leg->AddEntry((TObject*)0, Form("%2.1f < y < %2.1f", ymin, ymax), "");
  leg->AddEntry((TObject*)0, "", "");
  leg->AddEntry((TObject*)0, sigstr, "");
  if(strcmp(ggtxt,"")!=0) leg->AddEntry((TObject*)0, bkgstr, "");
  leg->AddEntry((TObject*)0, "", "");
  leg->AddEntry((TObject*)0, Form("M_{J/#psi} = %5.3f #pm %5.3f GeV", m0.getVal(), m0.getError()), "");
  leg->AddEntry((TObject*)0, Form("#sigma = %5.3f #pm %5.3f", sig.getVal(), sig.getError()), "");
  if(alphafix > 0) {leg->AddEntry((TObject*)0, Form("#alpha = %5.3f", alpha.getVal()), "");}
  else {            leg->AddEntry((TObject*)0, Form("#alpha = %5.3f #pm %5.3f", alpha.getVal(), alpha.getError()), "");}
  if(nfix > 0) {leg->AddEntry((TObject*)0, Form("n = %5.3f", n.getVal()), "");}
  else {        leg->AddEntry((TObject*)0, Form("n = %5.3f #pm %5.3f", n.getVal(), n.getError()), "");}
  if(strcmp(ggtxt,"")!=0) leg->AddEntry((TObject*)0, Form("#lambda = %5.3f #pm %5.3f", lambda.getVal(), lambda.getError()), "");
  if(lsPtYM) {
    Float_t lrn1[5] = {0.67, 0.1, 0.45, 0.07, -1.};  //legend range, x, dx, y, dy, margin
    TLegend *leg1 = mod->PrepareLegend(lrn1, 0.035);
    leg1->AddEntry(data, "unlike sign", "p");
    leg1->AddEntry(lsH, "like sign", "l");
  }

  can->cd(cpd); gPad->SetLeftMargin(lmg); gPad->SetBottomMargin(bmg);
  frame->Draw();// can->Update();
  if(lsPtYM) {lsH->Draw("same"); frame->Draw("same");}
  leg->Draw("same");
  if(lsPtYM) leg1->Draw("same");

ofstream zapis ("Preplot.csv", ios::app);
zapis<<Form("%.4f", alphafix)<<","<<vysl<<","<<err<<","<<nex<<","<<nexr<<","<<sigma<<endl;
zapis.close();

  //can->SaveAs("mass.pdf");

  //TCanvas *cMass = new TCanvas(name,name,600,600);
  //cMass->cd(1);

}//FitMass

//_____________________________________________________________________________
void FitPtJPsi(RooAbsData *dataIN,RooAbsData* bgIN,RooAbsData* ggIN,RooAbsData* signalIN, Double_t *nBgGG, char *name,RooAbsData* gPbIN)
//
// Fits pt ditribution of data with templates from background, gamma+gamma and signal
// The template data will be stored in histograms and from them the fit model will be created
//

{
  // reduce data to appropriate ranges
  pt.setMax(ptmax);
  char strReduce[120];
  sprintf(strReduce,"m>%f && m<%f && y>%f && y<%f && pt<%f",2.9, 3.2, ymin, ymax, ptmax);
  if(bgIN) RooDataSet* bg = bgIN->reduce(strReduce);
  RooDataSet* gg = ggIN->reduce(strReduce);
  RooDataSet* signal = signalIN->reduce(strReduce);
  RooDataSet* data = dataIN->reduce(strReduce);
  //RooDataSet *gPb = 0x0;
  //if(gPbIN) gPb = gPbIN->reduce(strReduce);
  if(gPbIN) RooDataSet *gPb = gPbIN->reduce(strReduce);

  Int_t nPtBins=nbinspT; // 25
  Double_t PtMax; PtMax = ptmax;

  // fill in histograms
  TH1F *bgH = new TH1F("bgH","bgH",nPtBins,0.0,ptmax);
  //TH1F *bgH = new TH1F("bgH","bgH",24,ptLow);
  if(bgIN) {for (Int_t i=0; i< bg->numEntries(); i++)  bgH->Fill(bg->get(i)->getRealValue("pt")  );}
  //bgH->Draw(); return;
  TH1F *ggH = new TH1F("ggH","ggH",nPtBins,0.0,PtMax);
  for (Int_t i=0; i< gg->numEntries(); i++) ggH->Fill(gg->get(i)->getRealValue("pt")  );
  TH1F *gPbH = new TH1F("gPbH","gPbH",nPtBins,0.0,PtMax);
  if(gPbIN) {for(Int_t i=0; i<gPb->numEntries(); i++) gPbH->Fill(gPb->get(i)->getRealValue("pt")  );}
  TH1F *signalH = new TH1F("signalH","signalH",nPtBins,0.0,PtMax);
  // TH1F *signalH = new TH1F("signalH","signalH",51,ptLow);
  for (Int_t i=0; i< signal->numEntries(); i++) signalH->Fill(signal->get(i)->getRealValue("pt")  );

  // create templates
  Int_t order = 0; // order of the interpolation between bins. Zero is no interpolation
  RooHistPdf ggPdf("ggPdf",   "ggPdf",  pt, RooDataHist("ggDH",  "ggDH",  RooArgList(pt),ggH),order);
  if(gPbIN) RooHistPdf gPbPdf("gPbPdf",   "gPbPdf",  pt, RooDataHist("gPbDH",  "gPbDH",  RooArgList(pt),gPbH),order);
  if(bgIN) {
    bgH->Smooth(50);
    RooHistPdf bgPdf("bgPdf",   "bgPdf",  pt, RooDataHist("bgDH",  "bgDH",  RooArgList(pt),bgH),1);
    RooDataSet *toyMCdata = bgPdf.generate(pt,100000);
    RooDataHist* hist1 = toyMCdata->binnedClone();
    //hist1->Draw(); return;
    RooHistPdf toybgPdf("toybgPdf","toybgPdf", RooArgList(pt),*hist1,order);
  }
  RooHistPdf signalPdf("signalPdf",   "signalPdf",  pt, RooDataHist("signalDH",  "signalDH",  RooArgList(pt),signalH),order); 

  // create the model
  //cout<<"#############################"<<endl;
  //cout<<"data->numEntries() "<<data->numEntries()<<endl;
  //cout<<"#############################"<<endl;
  RooRealVar ggN("ggN","number of gg",nBgGG[0],0,nBgGG[0]+2*nBgGG[1]); //200
  if(gPbIN) RooRealVar gPbN("gPbN","number of gPb",0.1*gPb->numEntries(),0,gPb->numEntries());
  if(bgIN) RooRealVar bgN("bgN","number of bg",0.1*bg->numEntries(),0,bg->numEntries());
  RooRealVar signalN("signalN","number of signal",0.4*data->numEntries(),0,data->numEntries());
  // Create the model as the sum of the three templates
  if(!gPbIN && !bgIN) {
    RooAddPdf Model("Model","Extended sum of gg, bk and signal",RooArgList(ggPdf,signalPdf),
      RooArgList(ggN,signalN));
  }
  if(gPbIN && !bgIN) {
    RooAddPdf Model("Model","Extended sum of gg, bk, gPb and signal",RooArgList(ggPdf,gPbPdf,signalPdf),
      RooArgList(ggN,gPbN,signalN));
  }
  if(!gPbIN && bgIN) {
    RooAddPdf Model("Model","Extended sum of gg, bk, gPb and signal",RooArgList(ggPdf,toybgPdf,signalPdf),
      RooArgList(ggN,bgN,signalN));
  }
  if(gPbIN && bgIN) {
    RooAddPdf Model("Model","Extended sum of gg, bk, gPb and signal",RooArgList(ggPdf,gPbPdf,toybgPdf,signalPdf),
      RooArgList(ggN,gPbN,bgN,signalN));
  }

  //RooAddPdf Model("Model","Extended sum of gg, bk and signal",RooArgList(ggPdf,toybgPdf),
    //RooArgList(ggN,bgN));

  // perform fit
  RooFitResult* r = Model.fitTo(*data,Extended(kTRUE),Save());

//gROOT->ProcessLine(".q");
  // plot
  //TCanvas *cPt = new TCanvas(name,name,600,600);
  //cPt->cd(1);
  //gPad->SetLeftMargin(0.15) ;
  TCanvas *cPt = c3;
  lmg = 0.1;
  bmg = 0.1;
  mod->AdjustCanvas(cPt, 1, 1, bmg, lmg);
  gPad->SetLeftMargin(lmg);
  RooPlot* frame = pt.frame(Bins(nPtBins),Title(plottitle)) ;
  frame->SetXTitle("dimuon p_{T} [GeV/c]");
  frame->GetYaxis()->SetTitleOffset(1.4);
  data->plotOn(frame,MarkerStyle(20),MarkerSize(1.));
  //Model.paramOn(frame,data);
  Model.plotOn(frame,Components(ggPdf), LineColor(kGreen), Range(0,5.0)    );
  if(bgIN) Model.plotOn(frame,Components(toybgPdf), LineColor(kRed), Range(0,5.0) );
  if(bgIN) Model.plotOn(frame,Components(bgPdf), LineColor(kRed), Range(0,5.0) );
  if(gPbIN) Model.plotOn(frame,Components(gPbPdf), LineColor(kViolet), Range(0,5.0)    );
  Model.plotOn(frame,Components(signalPdf), LineColor(kBlue), Range(0,5.0)  );
  Model.plotOn(frame,LineColor(kBlack), Range(0.,5.0) ) ;
  frame->Draw();
  //TLegend *leg1 = new TLegend(0.43,0.65,0.85,0.85); //0.35,0.60,0.85,0.85
  Float_t lrn[5] = {0.43, 0.3, 0.65, 0.22, -1.};  //legend range, x, dx, y, dy, margin
  TLegend *leg1 = mod->PrepareLegend(lrn, 0.035);
  leg1->SetFillStyle(0);
  leg1->AddEntry((TObject*)0, Form("%2.1f < y < %2.1f", ymin, ymax), "");
  leg1->AddEntry("Model_Norm[pt]_Range[0.000000_5.000000]", "Signal+Background+#gamma#gamma", "L");
  char tmpEntry[120];
  sprintf(tmpEntry,"Exclusive J/#psi: %.0f #pm %.0f", signalN.getVal(), signalN.getError());
  leg1->AddEntry("Model_Norm[pt]_Comp[signalPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  if(gPbIN) {
    sprintf(tmpEntry,"#gamma-Pb #rightarrow J/#psi: %.0f #pm %.0f", gPbN.getVal(), gPbN.getError());
    leg1->AddEntry("Model_Norm[pt]_Comp[gPbPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  }
  if(bgIN) {
    sprintf(tmpEntry,"Non-exclusive J/#psi: %.0f #pm %.0f", bgN.getVal(), bgN.getError());
    leg1->AddEntry("Model_Norm[pt]_Comp[toybgPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  }
  if(strcmp(bgtitle,"")!=0) leg1->AddEntry((TObject*)0, bgtitle, "");
  sprintf(tmpEntry,"#gamma#gamma #rightarrow #mu^{+}#mu^{-}: %.0f #pm %.0f", ggN.getVal(), ggN.getError());
  leg1->AddEntry("Model_Norm[pt]_Comp[ggPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  leg1->Draw();

  //inelastic fraction
  if(bgIN) Double_t fI = bgN.getVal()/signalN.getVal();
  //Double_t fIerr = fI*TMath::Sqrt( TMath::Power(bgN.getError()/bgN.getVal(),2.) + TMath::Power(signalN.getError()/signalN.getVal(),2.) );
  //binomial error in the inelastic fraction
  if(bgIN) Double_t fIerr = fI*TMath::Sqrt( (signalN.getVal()-bgN.getVal()) / (bgN.getVal()*signalN.getVal()) );

  cout<<"#################################################"<<endl;
  cout<<"      Excl J/psi:  "<<Form("%.0f", signalN.getVal())<<" +/- "<<Form("%.0f", signalN.getError());
  if(bgIN) cout<<" ,    Non-excl J/psi:  "<<Form("%.0f", bgN.getVal())<<" +/- "<<Form("%.0f", bgN.getError())<<endl;
  cout<<"      gamma-gamma:  "<<Form("%.0f", ggN.getVal())<<" +/- "<<Form("%.0f", ggN.getError())<<endl;
  //cout<<"      Sum:         "<<Form("%.0f", (signalN.getVal()+bgN.getVal()+ggN.getVal()) )<<endl;
  //cout<<"      fI:          "<<Form("%5.3f", fI)<<" +/- "<<Form("%5.3f", fIerr)<<endl;
  cout<<"#################################################"<<endl;

  cPt->Modified();
  cPt->Update();
  //char nameMacro[120];
  //sprintf(nameMacro,"MACROS/%s.C",name);
  //cPt->Print(nameMacro);
  cPt->SaveAs("pT.pdf");

}//FitPtJPsi

//_____________________________________________________________________________
void DrawInput(RooAbsData *dataIN) {

  pt.setMax(ptmax);
  char strReduce[120];
  //sprintf(strReduce,"m>%f && m<%f && y>%f && y<%f && pt<%f",2.9, 3.2, ymin, ymax, ptmax);
  sprintf(strReduce,"m>%f && m<%f && y>%f && y<%f && pt<%f",mmin, mmax, ymin, ymax, ptmax);
  RooDataSet* data = dataIN->reduce(strReduce);

  TH1F *dataH = new TH1F("dataH","dataH",nbinspT,0.0,ptmax);
  for(Int_t i=0; i<data->numEntries(); i++) dataH->Fill( data->get(i)->getRealValue("pt") );
  dataH->Smooth(50);
  RooHistPdf dataPdf("dataPdf",   "dataPdf",  pt, RooDataHist("dataDH",  "dataDH",  RooArgList(pt),dataH),1);
  RooDataSet *toyMCdata = dataPdf.generate(pt,100000);
  RooDataHist* hist1 = toyMCdata->binnedClone();
  RooHistPdf toybgPdf("toybgPdf","toybgPdf", RooArgList(pt),*hist1,0); 

  TCanvas *cPt = c3;
  lmg = 0.1;
  bmg = 0.1;
  mod->AdjustCanvas(cPt, 1, 1, bmg, lmg);
  gPad->SetLeftMargin(lmg);
  RooPlot* frame = pt.frame(Bins(nbinspT),Title(plottitle)) ;
  frame->SetXTitle("dimuon p_{T} [GeV/c]");
  frame->GetYaxis()->SetTitleOffset(1.4);

  data->plotOn(frame,MarkerStyle(20),MarkerSize(1.));
  //toyMCdata->plotOn(frame,MarkerStyle(20),MarkerSize(1.));
  //hist1->plotOn(frame,MarkerStyle(20),MarkerSize(1.));
  //toybgPdf.plotOn(frame,MarkerStyle(20),MarkerSize(1.));
  toybgPdf.plotOn(frame,Components(toybgPdf), LineColor(kRed), Range(0,5.0) );

  frame->Draw();

  Float_t lrn[5] = {0.7, 0.17, 0.65, 0.09, -1.};  //legend range, x, dx, y, dy, margin
  TLegend *leg1 = mod->PrepareLegend(lrn, 0.035);
  leg1->SetFillStyle(0);
  leg1->AddEntry(data, "data");
  leg1->AddEntry("toybgPdf_Norm[pt]_Comp[toybgPdf]_Range[0.000000_5.000000]", "toyMC", "L");

  Float_t lrn2[5] = {0.55, 0.17, 0.77, 0.1, -1.};  //legend range, x, dx, y, dy, margin
  TLegend *leg2 = mod->PrepareLegend(lrn2, 0.035);
  leg2->SetFillStyle(0);
  leg2->AddEntry((TObject*)0, Form("%2.1f < y < %2.1f", ymin, ymax), "");
  leg2->AddEntry((TObject*)0, Form("%2.1f < m_{#mu^{+}#mu^{-}} < %2.1f GeV", mmin, mmax), "");

  leg1->Draw();
  leg2->Draw();
  //cPt->cd(1)->SetLogy();
  cPt->Modified();
  cPt->Update();
  cPt->SaveAs("pT.pdf");

  cout<<"I-DrawInput, input data entries: "<<data->numEntries()<<endl;

  //gROOT->ProcessLine(".q");

}//DrawInput

//_____________________________________________________________________________
void FitPtGG(RooAbsData *dataIN,RooAbsData* bgIN,RooAbsData* ggIN,RooAbsData* lsIN, Double_t *nBgGG, char *name)
//
// Fits pt ditribution of data with templates from background and gamma+gamma
// The template data will be stored in histograms and from them the fit model will be created
//

{
  // reduce data to appropriate ranges
  char strReduce[120];
  //sprintf(strReduce,"m>%f && m<%f && pt<%f", mmin, mmax, ptmax);
  sprintf(strReduce,"m>%f && m<%f && y>%f && y<%f && pt<%f",mmin, mmax, ymin, ymax, ptmax);
  RooDataSet* bg = bgIN->reduce(strReduce);
  RooDataSet* gg = ggIN->reduce(strReduce);
  lsIN=0x0;
  if(lsIN) RooDataSet* ls = lsIN->reduce(strReduce);
  RooDataSet* data = dataIN->reduce(strReduce);

  Int_t nPtBins=nbinspT; // 30
  Double_t PtMax; PtMax = ptmax;

  // fill in histograms
  TH1F *bgH = new TH1F("bgH","bgH",nPtBins,0.0,ptmax);
  for (Int_t i=0; i< bg->numEntries(); i++)  bgH->Fill(bg->get(i)->getRealValue("pt")  );
  TH1F *ggH = new TH1F("ggH","ggH",nPtBins,0.0,PtMax);
  for (Int_t i=0; i< gg->numEntries(); i++) ggH->Fill(gg->get(i)->getRealValue("pt")  );
  TH1F *lsH = new TH1F("lsH","lsH",nPtBins,0.0,PtMax);
  if(lsIN) {
    for (Int_t i=0; i<ls->numEntries(); i++) lsH->Fill(ls->get(i)->getRealValue("pt")  );
  }

  // create templates
  Int_t order = 0; // order of the interpolation between bins. Zero is no interpolation
  RooHistPdf ggPdf("ggPdf",   "ggPdf",  pt, RooDataHist("ggDH",  "ggDH",  RooArgList(pt),ggH),order); 
  bgH->Smooth(50);
  RooHistPdf bgPdf("bgPdf",   "bgPdf",  pt, RooDataHist("bgDH",  "bgDH",  RooArgList(pt),bgH),1);
  RooDataSet *toyMCdata = bgPdf.generate(pt,100000);
  RooDataHist* hist1 = toyMCdata->binnedClone();
  RooHistPdf toybgPdf("toybgPdf","toybgPdf", RooArgList(pt),*hist1,order); 
  if(lsIN) {
    lsH->Smooth(50);
    RooHistPdf lsPdf("lsPdf",   "lsPdf",  pt, RooDataHist("lsDH",  "lsDH",  RooArgList(pt),lsH),1);
    RooDataSet *toyLS = lsPdf.generate(pt,100000);
    RooDataHist* hist1ls = toyLS->binnedClone();
    RooHistPdf toylsPdf("toylsPdf","toylsPdf", RooArgList(pt),*hist1ls,order); 
  }


  // create the model
  //cout<<"#############################"<<endl;
  //cout<<"data->numEntries() "<<data->numEntries()<<endl;
  //cout<<"#############################"<<endl;
  RooRealVar ggN("ggN","number of gg",nBgGG[0],0,nBgGG[0]+2*nBgGG[1]); //200
  RooRealVar bgN("bgN","number of bg",0.1*bg->numEntries(),0,bg->numEntries());
  if(lsIN) RooRealVar lsN("lsN","number of ls",0.1*ls->numEntries(),0,ls->numEntries());

  if(!lsIN) RooAddPdf Model("Model","Extended sum of gg and bg",RooArgList(ggPdf,toybgPdf), RooArgList(ggN,bgN));
  if( lsIN) RooAddPdf Model("Model","Extended sum of gg, bg and ls",RooArgList(ggPdf,toybgPdf,toylsPdf), RooArgList(ggN,bgN,lsN));

  // perform fit
  RooFitResult* r = Model.fitTo(*data,Extended(kTRUE),Save());

  // plot
  TCanvas *cPt = new TCanvas(name,name,600,600);
  cPt->cd(1);
  gPad->SetLeftMargin(0.15) ;
  RooPlot* frame = pt.frame(Bins(nPtBins),Title(plottitle)) ;
  frame->SetXTitle("dimuon p_{T} [GeV/c]");
  frame->GetYaxis()->SetTitleOffset(1.4) ;
  data->plotOn(frame,MarkerStyle(20),MarkerSize(1.));
  //Model.paramOn(frame,data);
  Model.plotOn(frame,Components(ggPdf), LineColor(kGreen), Range(0,5.0)    );
  Model.plotOn(frame,Components(toybgPdf), LineColor(kRed), Range(0,5.0) );
  if(lsIN) Model.plotOn(frame,Components(toylsPdf), LineColor(kViolet), Range(0,5.0) );
  //Model.plotOn(frame,Components(bgPdf), LineColor(kRed), Range(0,5.0) );
  //Model.plotOn(frame,Components(signalPdf), LineColor(kBlue), Range(0,5.0)  );
  Model.plotOn(frame,LineColor(kBlack), Range(0.,5.0) ) ;
  frame->Draw();
  //TLegend *leg1 = new TLegend(0.40,0.65,0.85,0.85); //0.35,0.60,0.85,0.85
  Float_t lrn[5] = {0.43, 0.3, 0.62, 0.25, -1.};  //legend range, x, dx, y, dy, margin
  TLegend *leg1 = mod->PrepareLegend(lrn, 0.035);
  leg1->SetFillStyle(0);
  leg1->AddEntry((TObject*)0, Form("%2.1f < y < %2.1f", ymin, ymax), "");
  leg1->AddEntry((TObject*)0, Form("%2.1f < m_{#mu^{+}#mu^{-}} < %2.1f GeV", mmin, mmax), "");
  //leg1->AddEntry((TObject*)0, "", "");
  leg1->AddEntry("Model_Norm[pt]_Range[0.000000_5.000000]", "Signal+Background", "L");
  char tmpEntry[120];
  //sprintf(tmpEntry,"Exclusive J/#psi: %d #pm %d", (Int_t) signalN.getVal(),(Int_t) signalN.getError());
  //leg1->AddEntry("Model_Norm[pt]_Comp[signalPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  sprintf(tmpEntry,"Elastic #gamma#gamma #rightarrow #mu^{+}#mu^{-}: %.0f #pm %.0f", ggN.getVal(), ggN.getError());
  leg1->AddEntry("Model_Norm[pt]_Comp[ggPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  sprintf(tmpEntry,"Non-exclusive #gamma#gamma: %.0f #pm %.0f", bgN.getVal(), bgN.getError());
  leg1->AddEntry("Model_Norm[pt]_Comp[toybgPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  if(strcmp(bgtitle,"")!=0) leg1->AddEntry((TObject*)0, bgtitle, "");
  leg1->Draw();

  cout<<"#################################################"<<endl;
  cout<<"      Elastic gamma-gamma:  "<<Form("%.0f", ggN.getVal())<<" +/- "<<Form("%.0f", ggN.getError())<<endl;
  cout<<"      Non-excl gamma-gamma:  "<<Form("%.0f", bgN.getVal())<<" +/- "<<Form("%.0f", bgN.getError())<<endl;
  //cout<<"      Sum:         "<<Form("%.0f", (signalN.getVal()+bgN.getVal()+ggN.getVal()) )<<endl;
  cout<<"#################################################"<<endl;

  cPt->Modified();
  cPt->Update();
  //char nameMacro[120];
  //sprintf(nameMacro,"MACROS/%s.C",name);
  //cPt->Print(nameMacro);
  cPt->SaveAs("pT.pdf");

}//FitPtGG















//_____________________________________________________________________________
void FitPtJPsi_gg(RooAbsData *dataIN,RooAbsData* bgIN,RooAbsData* ggIN,RooAbsData* signalIN, Double_t *nBgGG, char *name)
//
// Fits pt ditribution of data with templates from background, gamma+gamma and signal
// The template data will be stored in histograms and from them the fit model will be created
//

{
  // reduce data to appropriate ranges
  char strReduce[120];
  //sprintf(strReduce,"m>%f && m<%f && y>%f && y<%f && pt<%f",JPsiMassRangeMin,JPsiMassRangeMax,RapMin,RapMax,PtMax);
  sprintf(strReduce,"m>%f && m<%f && pt<%f", mmin, mmax, ptmax);
  //sprintf(strReduce,"m>%f && m<%f && pt<%f",1.8, 2.5,ptmax);
  //sprintf(strReduce,"m>%f && m<%f && pt<%f",2.,2.5,ptmax); //  2.,2.5 ok
  //sprintf(strReduce,"m>%f && m<%f && pt<%f", 3.5, 6., ptmax);
  RooDataSet* bg = bgIN->reduce(strReduce);
  RooDataSet* gg = ggIN->reduce(strReduce);
  RooDataSet* signal = signalIN->reduce(strReduce);
  RooDataSet* data = dataIN->reduce(strReduce);

  Int_t nPtBins=nbinspT; // 30
  Double_t PtMax; PtMax = ptmax;

  // fill in histograms
  TH1F *bgH = new TH1F("bgH","bgH",nPtBins,0.0,ptmax);
  //TH1F *bgH = new TH1F("bgH","bgH",24,ptLow);
  for (Int_t i=0; i< bg->numEntries(); i++)  bgH->Fill(bg->get(i)->getRealValue("pt")  );
  //bgH->Draw();
  TH1F *ggH = new TH1F("ggH","ggH",nPtBins,0.0,PtMax);
  // TH1F *ggH = new TH1F("ggH","ggH",51,ptLow);
  for (Int_t i=0; i< gg->numEntries(); i++) ggH->Fill(gg->get(i)->getRealValue("pt")  );
  //TH1F *signalH = new TH1F("signalH","signalH",nPtBins,0.0,PtMax);
  // TH1F *signalH = new TH1F("signalH","signalH",51,ptLow);
  //for (Int_t i=0; i< signal->numEntries(); i++) signalH->Fill(signal->get(i)->getRealValue("pt")  );

  // create templates
  Int_t order = 0; // order of the interpolation between bins. Zero is no interpolation
  RooHistPdf ggPdf("ggPdf",   "ggPdf",  pt, RooDataHist("ggDH",  "ggDH",  RooArgList(pt),ggH),order); 
  bgH->Smooth(50);
  RooHistPdf bgPdf("bgPdf",   "bgPdf",  pt, RooDataHist("bgDH",  "bgDH",  RooArgList(pt),bgH),1);
  RooDataSet *toyMCdata = bgPdf.generate(pt,100000);
  RooDataHist* hist1 = toyMCdata->binnedClone();
  RooHistPdf toybgPdf("toybgPdf","toybgPdf", RooArgList(pt),*hist1,order); 
  //RooHistPdf signalPdf("signalPdf",   "signalPdf",  pt, RooDataHist("signalDH",  "signalDH",  RooArgList(pt),signalH),order); 

  // create the model
  //cout<<"#############################"<<endl;
  //cout<<"data->numEntries() "<<data->numEntries()<<endl;
  //cout<<"#############################"<<endl;
  RooRealVar ggN("ggN","number of gg",nBgGG[0],0,nBgGG[0]+2*nBgGG[1]); //200
  RooRealVar bgN("bgN","number of bg",0.1*bg->numEntries(),0,bg->numEntries());
  //RooRealVar signalN("signalN","number of signal",0.4*data->numEntries(),0,data->numEntries());
  // Create the model as the sum of the three templates
  //RooAddPdf Model("Model","Extended sum of gg, bk and signal",RooArgList(ggPdf,toybgPdf,signalPdf),
  //  RooArgList(ggN,bgN,signalN));

  RooAddPdf Model("Model","Extended sum of gg, bk and signal",RooArgList(ggPdf,toybgPdf),
    RooArgList(ggN,bgN));

  // perform fit
  RooFitResult* r = Model.fitTo(*data,Extended(kTRUE),Save());

  // plot
  TCanvas *cPt = new TCanvas(name,name,600,600);
  cPt->cd(1);
  gPad->SetLeftMargin(0.15) ;
  RooPlot* frame = pt.frame(Bins(nPtBins),Title(plottitle)) ;
  frame->SetXTitle("dimuon p_{T} [GeV/c]");
  frame->GetYaxis()->SetTitleOffset(1.4) ;
  data->plotOn(frame,MarkerStyle(20),MarkerSize(1.));
  //Model.paramOn(frame,data);
  Model.plotOn(frame,Components(ggPdf), LineColor(kGreen), Range(0,5.0)    );
  Model.plotOn(frame,Components(toybgPdf), LineColor(kRed), Range(0,5.0) );
  Model.plotOn(frame,Components(bgPdf), LineColor(kRed), Range(0,5.0) );
  //Model.plotOn(frame,Components(signalPdf), LineColor(kBlue), Range(0,5.0)  );
  Model.plotOn(frame,LineColor(kBlack), Range(0.,5.0) ) ;
  frame->Draw();
  //TLegend *leg1 = new TLegend(0.40,0.65,0.85,0.85); //0.35,0.60,0.85,0.85
  Float_t lrn[5] = {0.43, 0.3, 0.65, 0.22, -1.};  //legend range, x, dx, y, dy, margin
  TLegend *leg1 = mod->PrepareLegend(lrn, 0.035);
  leg1->SetFillStyle(0);
  leg1->AddEntry((TObject*)0, Form("%2.1f < m_{#gamma#gamma} < %2.1f GeV", mmin, mmax), "");
  leg1->AddEntry("Model_Norm[pt]_Range[0.000000_5.000000]", "Signal+Background", "L");
  char tmpEntry[120];
  //sprintf(tmpEntry,"Exclusive J/#psi: %d #pm %d", (Int_t) signalN.getVal(),(Int_t) signalN.getError());
  //leg1->AddEntry("Model_Norm[pt]_Comp[signalPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  sprintf(tmpEntry,"Elastic #gamma#gamma #rightarrow #mu^{+}#mu^{-}: %.0f #pm %.0f", ggN.getVal(), ggN.getError());
  leg1->AddEntry("Model_Norm[pt]_Comp[ggPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  sprintf(tmpEntry,"Non-exclusive #gamma#gamma: %.0f #pm %.0f", bgN.getVal(), bgN.getError());
  leg1->AddEntry("Model_Norm[pt]_Comp[toybgPdf]_Range[0.000000_5.000000]", tmpEntry, "L");
  if(strcmp(bgtitle,"")!=0) leg1->AddEntry((TObject*)0, bgtitle, "");
  leg1->Draw();

  cout<<"#################################################"<<endl;
  cout<<"      Elastic gamma-gamma:  "<<Form("%.0f", ggN.getVal())<<" +/- "<<Form("%.0f", ggN.getError())<<endl;
  cout<<"      Non-excl gamma-gamma:  "<<Form("%.0f", bgN.getVal())<<" +/- "<<Form("%.0f", bgN.getError())<<endl;
  //cout<<"      Sum:         "<<Form("%.0f", (signalN.getVal()+bgN.getVal()+ggN.getVal()) )<<endl;
  cout<<"#################################################"<<endl;

  cPt->Modified();
  cPt->Update();
  //char nameMacro[120];
  //sprintf(nameMacro,"MACROS/%s.C",name);
  //cPt->Print(nameMacro);
  cPt->SaveAs("pT.pdf");

}//FitPtJPsi_gg

